import swarm.simtoolsgui.*;
import swarm.gui.*;
import swarm.space.*;
import swarm.activity.*;
import swarm.defobj.*;
import swarm.objectbase.*;
import swarm.*;

public class ObserverSwarm extends GUISwarmImpl{
	int displayFrequency;
	
	ActionGroup displayActions; // schedule data structs
	Schedule displaySchedule;
	
	Colormap colorMap; // allocate colours
	ZoomRaster worldRaster; // 2d display widget
	
	Model model; // the Swarm we're observing
	
	Value2dDisplay seaDisplay;
	Object2dDisplay fishDisplay; // display the heatbugs
	Object2dDisplay predatorDisplay;
	
	
	public ObserverSwarm(Zone aZone){
		super(aZone);
		displayFrequency=1;
	}
	
	public Object buildObjects(){
		super.buildObjects();
		
		model=new Model(Globals.env.globalZone);
		
		model.buildObjects();

		colorMap=new ColormapImpl(this);
		colorMap.setColor$ToName((byte)0,"black");
		colorMap.setColor$ToName((byte)1,"red");
		colorMap.setColor$ToName((byte)2,"green");
		
		worldRaster=new ZoomRasterImpl(this);
		worldRaster.setColormap(colorMap);
		worldRaster.setZoomFactor(4);
		worldRaster.setWidth$Height(
			model.getWorld().getSizeX(),
			model.getWorld().getSizeY());
		worldRaster.setWindowTitle("Sea Space");
		worldRaster.pack(); // draw the window.
		
		seaDisplay = new Value2dDisplayImpl(this,worldRaster,colorMap,model.getSea());
		
		try {
			fishDisplay = new Object2dDisplayImpl(
				this,
				worldRaster,
				model.getWorld(),
				new Selector(Class.forName("Fish"), "drawSelfOn", false));
		} catch (Exception e) {
			System.exit(1);
		}
		fishDisplay.setObjectCollection(model.getFishList());
		
		try {
			predatorDisplay = new Object2dDisplayImpl(
				this,
				worldRaster,
				model.getWorld2(),
				new Selector(Class.forName("Predator"), "drawSelfOn", false));
		} catch (Exception e) {
			System.exit(1);
		}
		predatorDisplay.setObjectCollection(model.getPredatorList());
		
		return this;
	}
	
	public Object buildActions(){
		
		// First, let our model swarm build its own schedule.
		
		model.buildActions();
		
		// Create an ActionGroup for display: a bunch of things that occur in
		// a specific order, but at one step of simulation time. Some of these
		// actions could be executed in parallel, but we don't explicitly
		// notate that here.
		
		displayActions=new ActionGroupImpl(this);
		
		// Schedule up the methods to draw the display of the world
		
		try {
			displayActions.createActionTo$message(seaDisplay,
				new Selector(seaDisplay.getClass(),"display",false));
			displayActions.createActionTo$message(fishDisplay,
				new Selector(fishDisplay.getClass(),"display",false));
			displayActions.createActionTo$message(predatorDisplay,
				new Selector(predatorDisplay.getClass(),"display",false));
			displayActions.createActionTo$message(worldRaster,
				new Selector(worldRaster.getClass(),"drawSelf",false));
			displayActions.createActionTo$message(getActionCache(),
				new Selector(getActionCache().getClass(),"doTkEvents",false));
		} catch (Exception e) {
			System.out.println ("Exception: " + e.getMessage ());
			System.exit(1);
		}
		
		// And the display schedule. Note the repeat interval is set from our
		// own Swarm data structure. Display is frequently the slowest part of a
		// simulation, so redrawing less frequently can be a help.
		
		displaySchedule = new ScheduleImpl(this,displayFrequency);
		displaySchedule.at$createAction(0,displayActions);
		
		return this;
    }
	
	public Activity activateIn(Swarm context){
    	super.activateIn(context);
		
		// Activate the model swarm in ourselves. The model swarm is a
		// subswarm of the observer swarm.
		
		model.activateIn(this);
		
		// Now activate our schedule in ourselves. This arranges for the
		// execution of the schedule we built.
		
		displaySchedule.activateIn(this);
		
		// Activate returns the swarm activity - the thing that's ready to run.
		
		return getActivity();
	}
	
	//fishHꂽv񐔂ƂHmo
	public void DeathCount(){
		model.DeathCount();
	}
}




















